/*
  NSPasteboard-NirAdditions.h

  HebrewService - system services for Hebrew users.
 
  Copyright (C) 2003-2006 Nir Soffer
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details:
  http://www.gnu.org/licenses/gpl.html
*/

#import <Cocoa/Cocoa.h>


@interface NSPasteboard (NSPasteboardNirAdditions)

/* Put Specific type on the pasteboard

Based on existing setMethods or specific objects pasteboard mehtods, like
NSColor -writeToPasteboard. Instead of reading the documentaion each time
just call this method and it will use the right method for the type for most
types.
*/
- (BOOL)setObject:(id)object forType:(NSString *)type;

/* Declare the type with nil owner, then invoke -setObject:forType: */
- (BOOL)putObject:(id)object forType:(NSString *)type;

/* Declare objects keys with nil owner, then invoke -setObect:forType: for each */
- (BOOL)putObjects:(NSDictionary *)objects;

/* Shortcut for puting strings for NSStringPboardType. */
- (BOOL)putString:(NSString *)string;

/* Put the best and most eficient representation for type

Extract the string for NSStringPboardType, or NSData for NSRTFPboardType and
NSRTFDPboardType. Ingore other types and return NO.
*/
- (BOOL)putAttributedString:(NSAttributedString *)attString
                    forType:(NSString *)type;
    
/* Get the object for specific type */
- (id)objectForType:(NSString *)type;

/* Return color object if one exists or nil */
- (NSColor *)readColor;

/* Return URL object if one exists or nil */
- (NSURL *)readURL;

/*
 Get an attributed string from the pasteboard for specific type. Simplify
 getting text from other applications, when we don't know if its plain text
 or rich text, or rich text with attachments.
 */
- (NSMutableAttributedString *)attributedStringForType:(NSString *)type;

@end
