/*!
    @header     NSString-FriBidiAdditions.h
    @abstract   Provides FriBidiAdditions category
    @copyright  2003-2006 Nir Soffer

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details:
    http://www.gnu.org/licenses/gpl.html
 */

#import <Cocoa/Cocoa.h>
#import "fribidi.h"

/*!
    @category   FriBidiAdditions
    @abstract   Provides FriBidi extenstions to NSString
    @discussion GNU FriBidi can order text in visual order implementing
                the unicode algorithm. This module uses a tiny subset of
                FriBidi to provide simple ordering of text.
 */

@interface NSString (FriBidiAdditions)

/*!
    @method     stringByOrderingVisuallyUsingDirection:
    @abstract   Reorder one line in visual order according to direction
    @discussion This method does not handle line breaking and meant to
                be called for single visual lines of a paragraphs. To
                handle multiple lines, use
                stringByOrderingLinesVisuallyUsingDirection:.
    @param      direction base writing direction
    @result     Returns an immutable autoreleased string or nil if there
                was a problem creating the object.
 */
- (NSString *)stringByOrderingVisuallyUsingDirection:
    (FriBidiCharType)direction;

/*!
    @method     stringByOrderingLinesVisuallyUsingDirection:
    @abstract   Reorder multiple lines in visual order according to direction
    @discussion Invoke stringByOrderingVisuallyUsingDirection: for each
                line in the reciever.
    @param      direction base writing direction
    @result     Returns an immutable autoreleased string or nil if there
                was a problem creating the object.
 */
- (NSString *)stringByOrderingLinesVisuallyUsingDirection:
    (FriBidiCharType)direction;

@end
