/*
    copyright 2003-2006 Nir Soffer

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details:
    http://www.gnu.org/licenses/gpl.html
*/

#import "NSString-FriBidiAdditions.h"
#import "NSString-NirAdditions.h"


@implementation NSString (FriBidiAdditions)

- (NSString *)stringByOrderingVisuallyUsingDirection:
    (FriBidiCharType)direction
{
    NSString *result = nil; // failure by default
    FriBidiChar *logical = NULL, *visual = NULL; // fribidi unicode buffers
    
    char *utf8String = (char *)[self UTF8String]; // utf8 buffer
    if (utf8String == NULL)
        goto cleanup;
    
    unsigned length = [self length];
    size_t bufferSize = (length + 1) * sizeof(FriBidiChar);
    
    logical = malloc(bufferSize);
    if (logical == NULL)
        goto cleanup;
    
    visual = malloc(bufferSize);
    if (visual == NULL)
        goto cleanup;    

    length = fribidi_utf8_to_unicode(utf8String, strlen(utf8String), logical);
    if (! fribidi_log2vis(logical, length, &direction, visual, 
                          NULL, NULL, NULL))
        goto cleanup;

    fribidi_unicode_to_utf8(visual, length, utf8String);
    result = [NSString stringWithUTF8String:utf8String];
    
cleanup:
    free(logical);
    free(visual);
    
    return result;
}


- (NSString *)stringByOrderingLinesVisuallyUsingDirection:
    (FriBidiCharType)direction
{
    NSMutableString *result = [[self mutableCopy] autorelease];
    unsigned maxIndex = [self length];
    unsigned lineStart = 0, lineEnd = 0, contentsEnd = 0;
    
    while (lineEnd < maxIndex) {
        NSRange range; // current line range
        NSString *line, *replacement; // temporary strings
        
        [self getLineStart:&lineStart 
                       end:&lineEnd 
               contentsEnd:&contentsEnd
                  forRange:NSMakeRange(lineEnd, 0)];

        if (lineStart == contentsEnd)
            continue; 
        
        range = NSMakeRange(lineStart, contentsEnd - lineStart);
        line = [self substringWithRange:range];
        replacement = [line stringByOrderingVisuallyUsingDirection:direction];
        if (replacement == nil)
            return nil;
        
        [result replaceCharactersInRange:range withString:replacement];
    }
    
    return result;
}

@end
